/* eslint-disable no-undef,no-unused-vars */
// noinspection JSUnresolvedVariable,JSUnresolvedFunction,CommaExpressionJS,JSUnusedGlobalSymbols,JSCheckFunctionSignatures

/*
 * 软件名称：waifus
 * 版本:V2.0.0
 * 版权：www.hell-vision.online
 * 开源协议：MIT
 */
(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
        typeof define === 'function' && define.amd ? define(factory) :
            (global = global || self, global.waifus = factory());
}(this, function () {
    'use strict';
    class waifusUtil {
        constructor() {}

        /**
         * 控制台监听辅助
         * @type {number}
         */
        static counts = 0

        /**
         * 判断对象是否为空
         * @param obj
         * @return {boolean}
         */
        static empty(obj) {
            return typeof obj == "undefined" || obj == null || obj === ""
        }

        /**
         * 获取随机String
         * @return {String}
         */
        static getRandText(text) {
            return Array.isArray(text) ? text[Math.floor(Math.random() * text.length + 1)-1] : text
        }

        /**
         * String格式化
         * @param context
         * @return {*}
         */
        static render(context) {
            let tokenReg = /(\\)?{([^{}\\]+)(\\)?}/g;
            return context.replace(tokenReg, function (word, slash1, token, slash2) {
                if (slash1 || slash2) { return word.replace('\\', ''); }
                let variables = token.replace(/\s/g, '').split('.');
                let currentObject = context;
                let i, length, variable;
                for (i = 0, length = variables.length; i < length; ++i) {
                    variable = variables[i];
                    currentObject = currentObject[variable];
                    if (currentObject === undefined || currentObject === null) return '';
                }
                return currentObject;
            });
        }

        /**
         * 针对Chrome打开控制台触发逻辑
         * @param fire
         * @return {(function(): void)|*}
         */
        static getChromeRerenderTestFunc = fire => {
            let mark = 0
            const currentCount = ++this.counts
            return () => {
                if (this.counts && this.counts !== currentCount) {
                    return
                }
                mark++
                if (mark === 2) {
                    this.counts = currentCount
                    fire()
                    mark = 1
                }
            }
        }

        /**
         * 监听控制台打开
         * @param fire
         */
        static getConsoleOpen = fire => {
            if (this.browserVersion().webKit) { // webkit内核：Chrome，Safari
                const re = /./
                re.toString = this.getChromeRerenderTestFunc(fire)
                const func = () => re
                func.toString = this.getChromeRerenderTestFunc(fire)
                // 注意这里是最新的
                const date = new Date()
                date.toString = this.getChromeRerenderTestFunc(fire)
                console.log('%c', /* < 92 */ func, /* 92 */ func(), /* 93 */ date)
            }else { // 兼容火狐
                const re = /./
                re.toString = fire
                console.log(re)
            }
        }

        /**
         * 获取浏览器类型
         * @return {{iPhone: boolean, webApp: boolean, trident: boolean, gecko: boolean, android: boolean, mobile: boolean, iPad: boolean, ios: boolean, presto: boolean, webKit: boolean}}
         */
        static browserVersion() {
            let u = navigator.userAgent
            return {
                trident: u.indexOf('Trident') > -1, //IE内核
                presto: u.indexOf('Presto') > -1, //opera内核
                webKit: u.indexOf('AppleWebKit') > -1, //苹果、谷歌内核
                gecko: u.indexOf('Firefox') > -1, //火狐内核Gecko
                mobile: !!u.match(/AppleWebKit.*Mobile.*/), //是否为移动终端
                ios: !!u.match(/\(i[^;]+;( U;)? CPU.+Mac OS X/), //ios
                android: u.indexOf('Android') > -1 || u.indexOf('Linux') > -1, //android
                iPhone: u.indexOf('iPhone') > -1 , //iPhone
                iPad: u.indexOf('iPad') > -1, //iPad
                webApp: u.indexOf('Safari') > -1 //Safari
            }
        }

        /**
         * 自建ajax
         * @param options
         * @return {Promise<unknown>}
         */
        static ajax(options = {}) {
            return new Promise((resolve, reject) => {
                waifusUtil.utilHttp({
                    ...options,
                    success: function(response){
                        //请求成功后执行的代码
                        resolve(response)
                    },
                    error:function(status){
                        //失败后执行的代码
                        reject(status)
                    }
                })
            })
        }

        /**
         * ajax请求
         * @param options
         */
        static utilHttp(options = {}){
            options.type = (options.type || 'GET').toUpperCase()
            options.dataType = options.dataType || 'json'
            let params = waifusUtil.formatParams(options.data)

            //创建-第一步
            let xhr
            //非IE6
            if(window.XMLHttpRequest){
                xhr = new XMLHttpRequest()
            }else{
                //ie6及其以下版本浏览器
                xhr = ActiveXObject('Microsoft.XMLHTTP')
            }

            //接收-第三步
            xhr.onreadystatechange = function(){
                if(xhr.readyState === 4){
                    let status=xhr.status
                    if(status >= 200 && status < 300){
                        options.success && options.success(xhr.responseText, xhr.responseXML)
                    }else{
                        options.error && options.error(status)
                    }
                }
            }

            try {
                //连接和发送-第二步
                if(options.type === 'GET'){
                    if (!options.static) {
                        xhr.open('GET', options.url + '?' + params, true)
                    }else {
                        xhr.open('GET', options.url, true)
                    }
                    xhr.send(null)
                }else if(options.type === 'POST'){
                    xhr.open('POST', options.url, true);
                    //设置表单提交时的内容类型
                    xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded")
                    xhr.send(params)
                }
            }catch (e) {
                console.log(e)
            }
        }

        /**
         * 格式化参数
         * @param data
         * @return {string}
         */
        static formatParams(data){
            let arr = [];
            for(let name in data){
                arr.push(encodeURIComponent(name) + '=' + encodeURIComponent(data[name]))
            }
            arr.push(('v=' + Math.random()).replace('.', ''))
            return arr.join('&')
        }
    }
    class waifus {
        /**
         * @param param {Object}
         */
        constructor(param = {}) {
            /**
             * 设置参数
             */
            this.live2d_settings = {
                getContainer: 'body', // 模型挂载位置
                elementId: 'waifu', // 模型区域id
                canvasId: 'live2d', // 画板id
                zIndex: '1000', // 默认层级
                modelAPI: 'http://service.hell-vision.online:3000/model/', // 自建 API 修改这里
                tipsMessage: 'http://service.hell-vision.online:3000/waifu-tips.json', // waifu-tips.json的位置
                hitokotoAPI: 'hitokoto', // 一言 API，可选 'lwl12.com', 'hitokoto.cn', 'jinrishici.
                modelId: 1, // 默认模型 ID，可在 F12 控制台找到
                modelTexturesId: 12, // 默认材质 ID，可在 F12 控制台找到
                showToolMenu: true, // 显示 工具栏          ，可选 true(真), false(假)
                canCloseLive2d: true, // 显示 关闭看板娘  按钮，可选 true(真), false(假)
                canSwitchModel: true, // 显示 模型切换    按钮，可选 true(真), false(假)
                canSwitchTextures: true, // 显示 材质切换    按钮，可选 true(真), false(假)
                canSwitchHitokoto: true, // 显示 一言切换    按钮，可选 true(真), false(假)
                canTakeScreenshot: true, // 显示 看板娘截图  按钮，可选 true(真), false(假)
                canTurnToHomePage: true, // 显示 返回首页    按钮，可选 true(真), false(假)
                canTurnToAboutPage: true, // 显示 跳转关于页  按钮，可选 true(真), false(假)
                modelStorage: true, // 记录 ID (刷新后恢复)，可选 true(真), false(假)
                modelRandMode: 'switch', // 模型切换，可选 'rand'(随机), 'switch'(顺序)
                modelTexturesRandMode: 'rand', // 材质切换，可选 'rand'(随机), 'switch'(顺序)
                showHitokoto: true, // 显示一言
                showF12Status: true, // 显示加载状态
                showF12Message: false, // 显示看板娘消息
                showF12OpenMsg: true, // 显示控制台打开提示
                showCopyMessage: true, // 显示 复制内容 提示
                showWelcomeMessage: true, // 显示进入面页欢迎词
                waifuSize: '400x357', // 看板娘大小，例如 '280x250', '600x535'
                waifuTipsSize: '380x100', // 提示框大小，例如 '250x70', '570x150'
                waifuFontSize: '20px', // 提示框字体，例如 '12px', '30px'
                waifuToolFont: '24px', // 工具栏字体，例如 '14px', '36px'
                waifuToolLine: '34px', // 工具栏行高，例如 '20px', '36px'
                waifuToolTop: '-40px', // 工具栏顶部边距，例如 '0px', '-60px'
                waifuMinWidth: '768px', // 面页小于 指定宽度 隐藏看板娘，例如 'disable'(禁用), '768px'
                waifuEdgeSide: 'right:0', // 看板娘贴边方向，例如 'left:0'(靠左 0px), 'right:30'(靠右 30px)
                waifuDraggable: 'disable', // 拖拽样式，例如 'disable'(禁用), 'axis-x'(只能水平拖拽), 'unlimited'(自由拖拽)
                waifuDraggableRevert: true, // 松开鼠标还原拖拽位置，可选 true(真), false(假), (当前版本不支持)
                l2dVersion: '2.0.0', // 当前版本
                l2dVerDate: '2022.03.20', // 版本更新日期
                homePageUrl: 'auto', // 主页地址，可选 'auto'(自动), '{URL 网址}'
                aboutPageUrl: '#', // 关于页地址, '{URL 网址}'
                screenshotCaptureName: 'live2d.png', // 看板娘截图文件名，例如 'live2d.png'
            }
            const live2d_settings = Object.assign(this.live2d_settings, param)
            /**
             * 提示框的计时器
             * @type {Number|null}
             */
            this.timer = null
            /**
             * url初始化
             */
            live2d_settings.homePageUrl = live2d_settings.homePageUrl === 'auto' ? window.location.protocol + '//' + window.location.hostname + '/' : live2d_settings.homePageUrl
            if (window.location.protocol === 'file:' && live2d_settings.modelAPI.substr(0, 2) === '//') live2d_settings.modelAPI = 'http:' + live2d_settings.modelAPI
            if (window.location.protocol === 'file:' && live2d_settings.tipsMessage.substr(0, 2) === '//') live2d_settings.tipsMessage = 'http:' + live2d_settings.tipsMessage
            /**
             * 添加容器到页面
             */
            this.drawing()
            /**
             * 模型容器
             * @type {Element}
             */
            this.container = document.querySelector('#' + this.live2d_settings.elementId)
            /**
             * 提示框对象
             * @type {Element}
             */
            this.tipMoodal = document.querySelector('.waifu-tips')
            /**
             * 工具栏对象
             * @type {Element}
             */
            this.toolModal = document.querySelector('.waifu-tool')
            /**
             * 获取提示语
             */
            waifusUtil.ajax({
                url: this.live2d_settings.tipsMessage,
                type: 'get',
                dataType: 'json',
                static: true,
            }).then(res => {
                this.tipMessage = JSON.parse(res)
                this.loadTipsMessage(this.tipMessage)
                this.loadWaifus(this.tipMessage)
            }).catch(err => {
                console.log('加载waifu-tips.json失败\n', err)
            })
            /**
             * 设置样式
             */
            live2d_settings.waifuSize = live2d_settings.waifuSize.split('x');
            live2d_settings.waifuTipsSize = live2d_settings.waifuTipsSize.split('x');
            live2d_settings.waifuEdgeSide = live2d_settings.waifuEdgeSide.split(':');
            this.canvas = document.querySelector('#' + this.live2d_settings.canvasId)
            this.canvas.setAttribute('width', live2d_settings.waifuSize[0])
            this.canvas.setAttribute('height', live2d_settings.waifuSize[1])
            this.tipMoodal.style.width = live2d_settings.waifuTipsSize[0] + 'px'
            this.tipMoodal.style.height = live2d_settings.waifuTipsSize[1] + 'px'
            this.tipMoodal.style.top = live2d_settings.waifuToolTop
            this.tipMoodal.style.fontSize = live2d_settings.waifuFontSize
            this.toolModal.style.fontSize = live2d_settings.waifuToolFont
            /**
             * 设置工具栏按钮
             */
            this.toolModal.childNodes.forEach(item => {
                if (item.nodeName === 'SPAN') {
                    item.style.lineHeight = live2d_settings.waifuToolLine
                }
            })
            /**
             * 设置live2d位置
             */
            if (live2d_settings.waifuEdgeSide[0] === 'left') {
                this.container.style.left = live2d_settings.waifuEdgeSide[1] + 'px'
            }else if (live2d_settings.waifuEdgeSide[0] === 'right') {
                this.container.style.right = live2d_settings.waifuEdgeSide[1] + 'px'
            }
            /**
             * 设置屏幕宽度过小时隐藏模型
             */
            window.waifuResize = () => {
                document.documentElement.clientWidth <= Number(live2d_settings.waifuMinWidth.replace('px', '')) ?
                    this.container.style.display = 'none' :
                    this.container.style.display = 'unset'
            };
            if (live2d_settings.waifuMinWidth !== 'disable') {
                window.waifuResize();
                window.addEventListener('resize', window.waifuResize)
            }
            /**
             * 如果设置了保存，从localStorage拿到上一次使用的id
             */
            let modelId, modelTexturesId
            if (this.live2d_settings.modelStorage) {
                modelId = localStorage.getItem('modelId') || this.live2d_settings.modelId;
                modelTexturesId = localStorage.getItem('modelTexturesId') || this.live2d_settings.modelTexturesId;
            } else {
                modelId = sessionStorage.getItem('modelId') || this.live2d_settings.modelId;
                modelTexturesId = sessionStorage.getItem('modelTexturesId') || this.live2d_settings.modelTexturesId;
            }
            /**
             * 加载live2d模型
             */
            this.loadModel(modelId, modelTexturesId)
            window.loadWaifu = (modelId = 1, modelTexturesId = 0) => {
                this.loadModel(modelId, modelTexturesId)
            }
        }

        /**
         * 创建HTML容器用于放置live2d模型
         */
        drawing() {
            /**
             * live2d模型的HTML结构
             * @type {string}
             */
            const live2d_canvas = '<div id="' + this.live2d_settings.elementId + '" class="waifu" style="z-index: ' + this.live2d_settings.zIndex + '">\n' +
            '        <div class="waifu-tips"></div>\n' +
            '        <canvas id="'+ this.live2d_settings.canvasId + '" class="live2d"></canvas>\n' +
            '        <div class="waifu-tool">\n' +
            '            <span class="fui-home"></span>\n' +
            '            <span class="fui-chat"></span>\n' +
            '            <span class="fui-eye"></span>\n' +
            '            <span class="fui-user"></span>\n' +
            '            <span class="fui-photo"></span>\n' +
            '            <span class="fui-info-circle"></span>\n' +
            '            <span class="fui-cross"></span>\n' +
            '        </div>\n' +
            '    </div>'
            const container = document.querySelector(this.live2d_settings.getContainer)
            const element = document.createElement('div')
            element.innerHTML = live2d_canvas
            container.append(element.firstElementChild)
            /**
             * 需要隐藏的内容
             * @type {Array}
             */
            const needHide = [
                { selector: '.waifu-tool', value: !this.live2d_settings.showToolMenu, },
                { selector: '.waifu-tool .fui-home', value: !this.live2d_settings.canTurnToHomePage, },
                { selector: '.waifu-tool .fui-chat', value: !this.live2d_settings.canSwitchHitokoto, },
                { selector: '.waifu-tool .fui-eye', value: !this.live2d_settings.canSwitchModel, },
                { selector: '.waifu-tool .fui-user', value: !this.live2d_settings.canSwitchTextures, },
                { selector: '.waifu-tool .fui-photo', value: !this.live2d_settings.canTakeScreenshot, },
                { selector: '.waifu-tool .fui-info-circle', value: !this.live2d_settings.canTurnToAboutPage, },
                { selector: '.waifu-tool .fui-cross', value: !this.live2d_settings.canCloseLive2d, },
            ]
            needHide.forEach(item => {
                if (item.value) {
                    document.querySelector(`#${this.live2d_settings.elementId} ${item.selector}`).style.display = 'none'
                }
            })
            /**
             * 控制台打印欢迎词
             */
            eval(function (p, a, c, k, e, r) {
                e = function (c) {
                    return (c < a ? '' : e(c / a)) + ((c = c % a) > 35 ? String.fromCharCode(c + 29) : c.toString(36))
                };
                if (!''.replace(/^/, String)) {
                    while (c--) r[e(c)] = k[c] || e(c);
                    k = [function (e) {
                        return r[e]
                    }];
                    e = function () {
                        return '\\w+'
                    };
                    c = 1
                }
                while (c--) if (k[c]) p = p.replace(new RegExp('\\b' + e(c) + '\\b', 'g'), k[c]);
                return p
            }('8.d(" ");8.d("\\U,.\\y\\5.\\1\\1\\1\\1/\\1,\\u\\2 \\H\\n\\1\\1\\1\\1\\1\\b \', !-\\r\\j-i\\1/\\1/\\g\\n\\1\\1\\1 \\1 \\a\\4\\f\'\\1\\1\\1 L/\\a\\4\\5\\2\\n\\1\\1 \\1 /\\1 \\a,\\1 /|\\1 ,\\1 ,\\1\\1\\1 \',\\n\\1\\1\\1\\q \\1/ /-\\j/\\1\\h\\E \\9 \\5!\\1 i\\n\\1\\1\\1 \\3 \\6 7\\q\\4\\c\\1 \\3\'\\s-\\c\\2!\\t|\\1 |\\n\\1\\1\\1\\1 !,/7 \'0\'\\1\\1 \\X\\w| \\1 |\\1\\1\\1\\n\\1\\1\\1\\1 |.\\x\\"\\1\\l\\1\\1 ,,,, / |./ \\1 |\\n\\1\\1\\1\\1 \\3\'| i\\z.\\2,,A\\l,.\\B / \\1.i \\1|\\n\\1\\1\\1\\1\\1 \\3\'| | / C\\D/\\3\'\\5,\\1\\9.\\1|\\n\\1\\1\\1\\1\\1\\1 | |/i \\m|/\\1 i\\1,.\\6 |\\F\\1|\\n\\1\\1\\1\\1\\1\\1.|/ /\\1\\h\\G \\1 \\6!\\1\\1\\b\\1|\\n\\1\\1\\1 \\1 \\1 k\\5>\\2\\9 \\1 o,.\\6\\2 \\1 /\\2!\\n\\1\\1\\1\\1\\1\\1 !\'\\m//\\4\\I\\g\', \\b \\4\'7\'\\J\'\\n\\1\\1\\1\\1\\1\\1 \\3\'\\K|M,p,\\O\\3|\\P\\n\\1\\1\\1\\1\\1 \\1\\1\\1\\c-,/\\1|p./\\n\\1\\1\\1\\1\\1 \\1\\1\\1\'\\f\'\\1\\1!o,.:\\Q \\R\\S\\T v"+e.V+" / W "+e.N);8.d(" ");', 60, 60, '|u3000|uff64|uff9a|uff40|u30fd|uff8d||console|uff8a|uff0f|uff3c|uff84|log|this.live2d_settings|uff70|u00b4|uff49||u2010||u3000_|u3008||_|___|uff72|u2500|uff67|u30cf|u30fc||u30bd|u4ece|u30d8|uff1e|__|u30a4|k_|uff17_|u3000L_|u3000i|uff1a|u3009|uff34|uff70r|u30fdL__||___i|l2dVerDate|u30f3|u30ce|nLive2D|u770b|u677f|u5a18|u304f__|l2dVersion|Hell Vision|u00b40i'.split('|'), 0, {}));
        }

        /**
         * 显示提示语的方法
         * @param text {String}
         * @param timeout {Number}
         * @param flag {Boolean}
         */
        showMessage(text, timeout, flag) {
            if (flag || sessionStorage.getItem('waifu-text') === '' || sessionStorage.getItem('waifu-text') === null) {
                if (Array.isArray(text)) text = text[Math.floor(Math.random() * text.length + 1) - 1];
                if (this.live2d_settings.showF12Message) console.log('[Message]', text.replace(/<[^<>]+>/g, ''));
                if (flag) sessionStorage.setItem('waifu-text', text);
                this.tipMoodal.innerHTML = text
                if (timeout === undefined) timeout = 5000;
                this.hideMessage(timeout);
            }
        }

        /**
         * 隐藏提示语的方法
         * @param timeout {Number}
         */
        hideMessage(timeout) {
            this.tipMoodal.style.opacity = 1
            if (timeout === undefined) timeout = 5000;
            if (this.timer) {
                clearTimeout(this.timer)
                this.timer = null
            }
            this.timer = window.setTimeout(() => {
                sessionStorage.removeItem('waifu-text')
                this.tipMoodal.style.opacity = 0
                clearTimeout(this.timer)
                this.timer = null
            }, timeout);
        }

        /**
         * 加载模型的方法
         * @param modelId {Number}
         * @param modelTexturesId {Number}
         */
        loadModel(modelId = 1, modelTexturesId = 0) {
            if (this.live2d_settings.modelStorage) {
                localStorage.setItem('modelId', modelId);
                localStorage.setItem('modelTexturesId', modelTexturesId);
            } else {
                sessionStorage.setItem('modelId', modelId);
                sessionStorage.setItem('modelTexturesId', modelTexturesId);
            }
            window.loadlive2d(this.live2d_settings.canvasId,
                // url ? url : `../../static/api/model/Potion-Maker/Pio/index.json`,
                this.live2d_settings.modelAPI + 'get/?id=' + modelId + '-' + modelTexturesId,
                (this.live2d_settings.showF12Status ? console.log('[Status]', 'live2d', '模型', modelId + '-' + modelTexturesId, '加载完成') : null));
        }

        /**
         * 加载提示文本
         * @param result {Object|String}
         */
        loadTipsMessage(result) {
            window.waifu_tips = result
            /**
             * 统一用that
             * @type {waifus}
             */
            const that = this
            /**
             * 鼠标悬浮事件
             */
            result.mouseover && result.mouseover.forEach(item => {
                const element = document.querySelector(`#${that.live2d_settings.elementId} ${item.selector}`)
                element && element.addEventListener('mouseover', () => {
                    let text = waifusUtil.getRandText(item.text)
                    text = waifusUtil.render(text)
                    that.showMessage(text, 3000)
                })
            })
            /**
             * 点击模型事件
             */
            result.click && result.click.forEach(item => {
                const element = document.querySelector(`#${that.live2d_settings.elementId} #${that.live2d_settings.canvasId}`)
                element && element.addEventListener('click', () => {
                    let text = waifusUtil.getRandText(item.text)
                    text = waifusUtil.render(text)
                    that.showMessage(text, 3000)
                })
            })
            /**
             * 特殊提示语
             * 根据打开页面的时间显示提示
             */
            result.seasons && result.seasons.forEach(tips => {
                let now = new Date()
                let after = tips.date.split('-')[0]
                let before = tips.date.split('-')[1] || after
                if ((after.split('/')[0] <= now.getMonth() + 1 && now.getMonth() + 1 <= before.split('/')[0]) &&
                    (after.split('/')[1] <= now.getDate() && now.getDate() <= before.split('/')[1])) {
                    let text = waifusUtil.getRandText(tips.text)
                    text = waifusUtil.render(text)
                    that.showMessage(text, 6000, true)
                }
            })
            /**
             * 控制台打开监听
             */
            if (that.live2d_settings.showF12OpenMsg) {
                waifusUtil.getConsoleOpen(() => {
                    that.showMessage(waifusUtil.getRandText(result.waifu.console_open_msg), 5000, true)
                })
            }
            /**
             * 复制时提示
             */
            if (that.live2d_settings.showCopyMessage) {
                document.addEventListener('copy', () => {
                    that.showMessage(waifusUtil.getRandText(result.waifu.copy_message), 5000, true);
                })
            }
            /**
             * 拍照方法
             */
            document.querySelector(`#${that.live2d_settings.elementId} .waifu-tool .fui-photo`).addEventListener('click', () => {
                that.showMessage(waifusUtil.getRandText(result.waifu.screenshot_message), 5000, true);
                window.Live2D.captureName = that.live2d_settings.screenshotCaptureName;
                window.Live2D.captureFrame = true;
            })
            /**
             * 关闭live2d模型方法
             */
            document.querySelector(`#${that.live2d_settings.elementId} .waifu-tool .fui-cross`).addEventListener('click', () => {
                sessionStorage.setItem('waifu-display', 'none');
                that.showMessage(waifusUtil.getRandText(result.waifu.hidden_message), 1300, true);
                window.setTimeout(function () {
                    document.querySelector(`#${that.live2d_settings.elementId}`).style.display = 'none'
                }, 1300);
            })
            /**
             * 回到首页
             */
            document.querySelector(`#${that.live2d_settings.elementId} .waifu-tool .fui-home`).addEventListener('click', () => {
                window.location = live2d_settings.homePageUrl;
            })
            /**
             * 打开live2d信息页
             */
            document.querySelector(`#${that.live2d_settings.elementId} .waifu-tool .fui-info-circle`).addEventListener('click', () => {
                window.open(live2d_settings.aboutPageUrl);
            })
            /**
             * 给window绑定隐藏和显示的方法
             */
            window.closeWaifu = () => {
                sessionStorage.setItem('waifu-display', 'none');
                that.showMessage(waifusUtil.getRandText(result.waifu.hidden_message), 1300, true);
                window.setTimeout(function () {
                    document.querySelector(`#${that.live2d_settings.elementId}`).style.display = 'none'
                }, 1300);
            }
            window.openWaifu = () => {
                sessionStorage.setItem('waifu-display', 'unset');
                this.showMessage(waifusUtil.getRandText(result.waifu.show_message), 1300, true);
                document.querySelector(`#${that.live2d_settings.elementId}`).style.display = 'unset'
            }
            /**
             * 欢迎语
             * @param result
             */
            window.showWelcomeMessage = function (result) {
                let text;
                if (window.location.href === that.live2d_settings.homePageUrl) {
                    let now = (new Date()).getHours();
                    if (now > 23 || now <= 5) text = waifusUtil.getRandText(result.waifu.hour_tips['t23-5']);
                    else if (now > 5 && now <= 7) text = waifusUtil.getRandText(result.waifu.hour_tips['t5-7']);
                    else if (now > 7 && now <= 11) text = waifusUtil.getRandText(result.waifu.hour_tips['t7-11']);
                    else if (now > 11 && now <= 14) text = waifusUtil.getRandText(result.waifu.hour_tips['t11-14']);
                    else if (now > 14 && now <= 17) text = waifusUtil.getRandText(result.waifu.hour_tips['t14-17']);
                    else if (now > 17 && now <= 19) text = waifusUtil.getRandText(result.waifu.hour_tips['t17-19']);
                    else if (now > 19 && now <= 21) text = waifusUtil.getRandText(result.waifu.hour_tips['t19-21']);
                    else if (now > 21 && now <= 23) text = waifusUtil.getRandText(result.waifu.hour_tips['t21-23']);
                    else text = waifusUtil.getRandText(result.waifu.hour_tips.default);
                } else {
                    let referrer_message = result.waifu.referrer_message;
                    if (document.referrer !== '') {
                        let referrer = document.createElement('a');
                        referrer.href = document.referrer;
                        let domain = referrer.hostname.split('.')[1];
                        if (window.location.hostname === referrer.hostname)
                            text = referrer_message.localhost[0] + document.title.split(referrer_message.localhost[2])[0] + referrer_message.localhost[1];
                        else if (domain === 'baidu')
                            text = referrer_message.baidu[0] + referrer.search.split('&wd=')[1].split('&')[0] + referrer_message.baidu[1];
                        else if (domain === 'so')
                            text = referrer_message.so[0] + referrer.search.split('&q=')[1].split('&')[0] + referrer_message.so[1];
                        else if (domain === 'google')
                            text = referrer_message.google[0] + document.title.split(referrer_message.google[2])[0] + referrer_message.google[1];
                        else {
                            for (const key in result.waifu.referrer_hostname) {
                                if (key === referrer.hostname) {
                                    referrer.hostname = waifusUtil.getRandText(result.waifu.referrer_hostname[key])
                                }
                            }
                            text = referrer_message.default[0] + referrer.hostname + referrer_message.default[1]
                        }
                    } else text = referrer_message.none[0] + document.title.split(referrer_message.none[2])[0] + referrer_message.none[1]
                }
                that.showMessage(text, 6000)
            }
            if (that.live2d_settings.showWelcomeMessage) window.showWelcomeMessage(result)
        }

        /**
         * 加载模型的各种交互
         * @param result {Object|String}
         */
        loadWaifus(result) {
            let waifu_tips = result.waifu;
            let that = this;

            /* 加载其他模型 */
            function loadOtherModel() {
                let modelId = modelStorageGetItem('modelId') || this.live2d_settings.modelId
                let modelRandMode = that.live2d_settings.modelRandMode

                waifusUtil.ajax({
                    cache: modelRandMode === 'switch',
                    url: that.live2d_settings.modelAPI + modelRandMode,
                    dataType: "json",
                    data: {
                        id: modelId
                    }
                }).then(res => {
                    res = JSON.parse(res)
                    that.loadModel(res.model['id'])
                    let message = res.model['message']
                    for (const key in waifu_tips.model_message) {
                        if (key === res.model['id']) {
                            message = waifusUtil.getRandText(waifu_tips.model_message[key])
                        }
                    }
                    that.showMessage(message, 3000, true)
                }).catch(() => {
                    console.log('切换模型失败')
                    that.showMessage('切换模型失败', 3000, true)
                })
            }

            /* 加载随机材质 */
            function loadRandTextures() {
                let modelId = modelStorageGetItem('modelId') || this.live2d_settings.modelId
                let modelTexturesId = modelStorageGetItem('modelTexturesId') || this.live2d_settings.modelTexturesId
                let modelTexturesRandMode = that.live2d_settings.modelTexturesRandMode

                waifusUtil.ajax({
                    cache: modelTexturesRandMode === 'switch',
                    url: that.live2d_settings.modelAPI + modelTexturesRandMode + '_textures',
                    dataType: "json",
                    data: {
                        id: modelId + '-' + modelTexturesId
                    }
                }).then(res => {
                    res = JSON.parse(res)
                    if (res.textures['id'] === 0 && (modelTexturesId === '1' || modelTexturesId === '0')) {
                        that.showMessage(waifu_tips.load_rand_textures[0], 3000, true)
                    }
                    else {
                        that.showMessage(waifu_tips.load_rand_textures[1], 3000, true)
                    }
                    that.loadModel(modelId, res.textures['id'])
                }).catch((err) => {
                    console.log('切换材质失败', err)
                    that.showMessage('切换材质失败', 3000, true)
                })
            }

            /* 获取LocalStorage */
            function modelStorageGetItem(key) {
                return that.live2d_settings.modelStorage ? localStorage.getItem(key) : sessionStorage.getItem(key);
            }

            /* 检测用户活动状态，并在空闲时显示一言 */
            if (that.live2d_settings.showHitokoto) {
                window.getActed = false
                window.hitokotoTimer = 0
                window.hitokotoInterval = false
                document.addEventListener('mousemove', () => {
                    window.getActed = true
                })
                document.addEventListener('keydown', () => {
                    window.getActed = true
                })
                window.setInterval(function () {
                    if (!window.getActed) {
                        ifActed()
                    } else {
                        elseActed()
                    }
                }, 1000)
            }

            /* 非活动状态 */
            function ifActed() {
                if (!window.hitokotoInterval) {
                    window.hitokotoInterval = true;
                    window.hitokotoTimer = window.setInterval(showHitokotoActed, 30000);
                }
            }

            /* 活动状态 */
            function elseActed() {
                window.getActed = window.hitokotoInterval = false;
                window.clearInterval(hitokotoTimer);
            }

            /* 显示一言 */
            function showHitokotoActed() {
                if (document.visibilityState === 'visible') showHitokoto();
            }

            /* 一言API */
            function showHitokoto() {
                switch (that.live2d_settings.hitokotoAPI) {
                    case 'lwl12.com':
                        waifusUtil.ajax({
                            url: 'https://api.lwl12.com/hitokoto/v1',
                            data: {
                                encode: 'realjson'
                            }
                        }).then(function (result) {
                            if (!waifusUtil.empty(result.source)) {
                                let text = waifu_tips.hitokoto_api_message['lwl12.com'][0];
                                if (!waifusUtil.empty(result.author)) text += waifu_tips.hitokoto_api_message['lwl12.com'][1];
                                text = waifusUtil.render(text)
                                // text = text.render({source: result.source, creator: result.author});
                                window.setTimeout(function () {
                                    that.showMessage(text + waifu_tips.hitokoto_api_message['lwl12.com'][2], 3000, true);
                                }, 5000);
                            }
                            that.showMessage(result.text, 5000, true);
                        });
                        break;
                    case 'fghrsh.net':
                        waifusUtil.ajax({
                            url: 'https://api.fghrsh.net/hitokoto/rand/',
                            data: {
                                encode: 'jsc',
                                uid: '3335'
                            }
                        }).then(function (result) {
                            result = JSON.parse(result)
                            that.showMessage(result.hitokoto, 5000, true);
                        });
                        break;
                    case 'jinrishici.com':
                        waifusUtil.ajax({
                            url: 'https://v2.jinrishici.com/one.json',
                            xhrFields: {withCredentials: true},
                        }).then(function (result) {
                            result = JSON.parse(result)
                            that.showMessage(result.data.content, 5000, true);
                        });
                        break;
                    default:
                        waifusUtil.ajax({
                            url: 'https://v1.hitokoto.cn',
                        }).then(function (result) {
                            result = JSON.parse(result)
                            that.showMessage(result.hitokoto, 5000, true);
                        });
                }
            }

            // 绑定事件
            document.querySelector(`#${that.live2d_settings.elementId} .waifu-tool .fui-eye`)
                .addEventListener('click', loadOtherModel)
            document.querySelector(`#${that.live2d_settings.elementId} .waifu-tool .fui-user`)
                .addEventListener('click', loadRandTextures)
            document.querySelector(`#${that.live2d_settings.elementId} .waifu-tool .fui-chat`)
                .addEventListener('click', showHitokoto)
        }
    }
    return waifus;
}));
